#!/bin/bash

# hunter_seeker.sh -- a script to blacklist spammers by rDNS name
# Copyright (C) 2013 Sam Clippinger (samc (at) silence (dot) org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

MAILLOG_FILE=/var/log/maillog
BLACKLIST_DIRECTORY=/home/vpopmail/blacklist_rdns.d
UNBLACKLISTABLE_DOMAINS=/home/vpopmail/unblacklistable_domains
CURL_CONNECT_TIMEOUT_SECS=5
SITE_DIRECTORY=/home/vpopmail/traps/hunter_seeker/sites
HEADER_DIRECTORY=/home/vpopmail/traps/hunter_seeker/headers
IP_DIRECTORY=/home/vpopmail/traps/hunter_seeker/ips
FILTER_DIRECTORY=/home/vpopmail/traps/hunter_seeker/filters
SITE_LIFETIME_MINS=2800
RUN_FREQUENCY_MINS=120
SEEK_LIFETIME_MINS=40320
MAX_REDIRECTS=5
AGENT_STRING="Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)"
LOCK_FILE=/home/vpopmail/traps/hunter_seeker/hunter_seeker.lock
SEEK_LOCK_FILE=/home/vpopmail/traps/hunter_seeker/hunter_seeker.seek.lock
DOMAIN2PATH=/usr/local/bin/domain2path

check_site() {
  output=`grep -E "^$2$" ${UNBLACKLISTABLE_DOMAINS}`
  if [ -z "${output}" ]
  then
    for filter in ${FILTER_DIRECTORY}/*
    do
      blacklisted=""
      source ${filter} $1 $2 $3
      if [ "${blacklisted}" != "" ]
      then
        touch ${filter}
        return
      fi
    done
  fi
}

search_ips() {
  if [ ! -z "$1" ]
  then
    local a=`echo $1 | cut -f1 -d.`
    local b=`echo $1 | cut -f2 -d.`
    local c=`echo $1 | cut -f3 -d.`

    if [ "${a}${b}${c}" != "" ]
    then
      local d=0
      local name_list=""
      while [ ${d} -lt 256 ]
      do
        local name_list="${name_list} `dnsptr ${a}.${b}.${c}.${d}`"
        local d=$[$d+1]
      done

      for si_name in `echo ${name_list} | domainsplit | sort | uniq`
      do
        if [ ! -z "${si_name}" ]
        then
          local si_base_name=`echo ${si_name} | domainsplit`
          blacklisted=""

          if [ ! -f ${HEADER_DIRECTORY}/${si_base_name} ]
          then
            if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}` ]
            then
              if [ "${si_name}" != "www.${si_base_name}" ]
              then
                curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://www.${si_base_name}/ -o ${SITE_DIRECTORY}/www.${si_base_name} --dump-header ${HEADER_DIRECTORY}/www.${si_base_name} --speed-limit 10 --speed-time 90
                check_site www.${si_base_name} ${si_base_name}
              fi
            else
              touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}`
            fi

            if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}` ]
            then
              if [ "${si_name}" != "${si_base_name}" ]
              then
                curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://${si_base_name}/ -o ${SITE_DIRECTORY}/${si_base_name} --dump-header ${HEADER_DIRECTORY}/${si_base_name} --speed-limit 10 --speed-time 90
                check_site ${si_base_name} ${si_base_name}
              fi
            else
              touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}`
            fi

            if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}` ]
            then
              if [ ! -f ${HEADER_DIRECTORY}/${si_name} ]
              then
                curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://${si_name}/ -o ${SITE_DIRECTORY}/${si_name} --dump-header ${HEADER_DIRECTORY}/${si_name} --speed-limit 10 --speed-time 90
                check_site ${si_name} ${si_base_name}
              fi
            else
              touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${si_base_name}`
            fi

            if [ ! -z "${blacklisted}" ]
            then
              touch ${IP_DIRECTORY}/${a}.${b}.${c}
            fi
          fi
        fi
      done
    fi
  fi
}


mkdir -p ${SITE_DIRECTORY}
mkdir -p ${HEADER_DIRECTORY}
mkdir -p ${IP_DIRECTORY}

if [ -z "$1" ]
then
  if [ ! -f ${LOCK_FILE} ]
  then
    touch ${LOCK_FILE}

    find ${SITE_DIRECTORY} ${HEADER_DIRECTORY} -mmin +${SITE_LIFETIME_MINS} -print0 | xargs -0 rm -f

    for line in `grep spamdyke ${MAILLOG_FILE} 2> /dev/null | timefilter $[${RUN_FREQUENCY_MINS}*60] 0 | awk '{ print $12 ":" $14 }' | sort | uniq`
    do
      name=`echo ${line} | cut -f2 -d:`
      if [ "${name}" != "(unknown)" ]
      then
        ip=`echo ${line} | cut -f1 -d:`
        ip_a=`echo ${ip} | cut -f1 -d.`
        ip_b=`echo ${ip} | cut -f2 -d.`
        ip_c=`echo ${ip} | cut -f3 -d.`
        base_name=`echo ${line} | cut -f2 -d: | domainsplit`

        if [ ! -z "${base_name}" ]
        then
          if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}` ]
          then
            if [ ! -f ${HEADER_DIRECTORY}/${base_name} ]
            then
              if [ "${name}" != "(unknown)" ]
              then
                if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}` ]
                then
                  if [ "${name}" != "www.${base_name}" ]
                  then
                    curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://www.${base_name}/ -o ${SITE_DIRECTORY}/www.${base_name} --dump-header ${HEADER_DIRECTORY}/www.${base_name} --speed-limit 10 --speed-time 90
                    check_site www.${base_name} ${base_name} ${ip_a}.${ip_b}.${ip_c}
                  fi
                else
                  touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}`
                fi

                if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}` ]
                then
                  if [ "${name}" != "${base_name}" ]
                  then
                    curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://${base_name}/ -o ${SITE_DIRECTORY}/${base_name} --dump-header ${HEADER_DIRECTORY}/${base_name} --speed-limit 10 --speed-time 90
                    check_site ${base_name} ${base_name} ${ip_a}.${ip_b}.${ip_c}
                  fi
                else
                  touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}`
                fi

                if [ ! -f ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}` ]
                then
                  if [ ! -f ${HEADER_DIRECTORY}/${name} ]
                  then
                    curl --compressed --user-agent "${AGENT_STRING}" --silent --location --max-redirs ${MAX_REDIRECTS} --connect-timeout ${CURL_CONNECT_TIMEOUT_SECS} http://${name}/ -o ${SITE_DIRECTORY}/${name} --dump-header ${HEADER_DIRECTORY}/${name} --speed-limit 10 --speed-time 90
                    check_site ${name} ${base_name} ${ip_a}.${ip_b}.${ip_c}
                  fi
                else
                  touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}`
                fi
              fi
            fi
          else
            touch ${BLACKLIST_DIRECTORY}/`${DOMAIN2PATH} ${base_name}`
          fi
        fi
      fi
    done

    rm -f ${LOCK_FILE}
  fi
else
  if [ "$1" == "rescan" ]
  then
    if [ ! -f ${LOCK_FILE} ]
    then
      touch ${LOCK_FILE}

      pushd ${HEADER_DIRECTORY} > /dev/null
      for file in *
      do
        if [ "${file}" != "*" ]
        then
          base_name=`echo ${file} | domainsplit`
          check_site ${file} ${base_name}
        fi
      done
      popd > /dev/null

      rm -f ${LOCK_FILE}
    fi
  else
    if [ "$1" == "seek" ]
    then
      if [ ! -f ${SEEK_LOCK_FILE} ]
      then
        touch ${SEEK_LOCK_FILE}

        rm -f `find ${IP_DIRECTORY} -mmin +${SEEK_LIFETIME_MINS} -print`

        pushd ${IP_DIRECTORY} > /dev/null
        for ip in *
        do
          if [ "${ip}" != "*" ]
          then
            search_ips ${ip}
          fi
        done
        popd > /dev/null

        rm -f ${SEEK_LOCK_FILE}
      fi
    else
      if [ ! -f ${LOCK_FILE} ]
      then
        touch ${LOCK_FILE}

        search_ips $1

        rm -f ${LOCK_FILE}
      fi
    fi
  fi
fi
